#include "simwin.h"
#include "win1820.h"

extern int SetupDirty;

DEFINE_RESPONSE_TABLE1(TSimWindow, TFrameWindow)
	EV_WM_TIMER,
	EV_WM_LBUTTONDOWN,
END_RESPONSE_TABLE;

int	SimROMx[4] = { 51*SS_BM+2, 141*SS_BM+2, 231*SS_BM+2, 316*SS_BM+2 };
int	SimROMy[4] = { 15*SS_BM,  43*SS_BM,  14*SS_BM,  41*SS_BM };

TSimWindow::TSimWindow(TWindow* parent, const char* title, ThermBus* MainBus,
	TButtonGadget* cs_bg, TButtonGadget* be_bg, BOOL* del_cs)
  : TFrameWindow(parent, title)
{
	TClientDC winDC(*this);
	TMemoryDC stretchedDC(winDC);
	TMemoryDC memDC(winDC);

	SimBus = MainBus;

	dltd = del_cs;
	*dltd = FALSE;

	sw_cs_bg = cs_bg;
	sw_be_bg = be_bg;

	sw_cs_bg->SetButtonState( TButtonGadget::Indeterminate );
	sw_be_bg->SetButtonState( TButtonGadget::Indeterminate );

	// load bitmap
	//
	SimBitmap = new TBitmap(*GetApplication(), "SIM_BM");

	AlarmBitmap = new TBitmap(*GetApplication(), "SIM_ALARM_BM" );

	NewSimBitmap = new TBitmap(winDC,
		SimBitmap->Width()*SS_BM, SimBitmap->Height()*SS_BM);

	stretchedDC.SelectObject(*NewSimBitmap);
	memDC.SelectObject(*SimBitmap);
	stretchedDC.StretchBlt(0, 0,
		NewSimBitmap->Width(), NewSimBitmap->Height(), memDC, 0, 0,
		SimBitmap->Width(), SimBitmap->Height(), SRCCOPY);

	// Set the window size to size of bitmap plus non-client area size
	//
	Attr.X = GetSystemMetrics( SM_CXSCREEN )/2 - NewSimBitmap->Width()/2;
	Attr.Y = GetSystemMetrics( SM_CYSCREEN )/2 - NewSimBitmap->Height()/2;
	Attr.W = NewSimBitmap->Width() + 2*GetSystemMetrics(SM_CXBORDER);
	Attr.H = NewSimBitmap->Height() + GetSystemMetrics(SM_CYBORDER) +
												 GetSystemMetrics(SM_CYCAPTION);

	Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION;

	TempFont = new TFont( "MS Sans Serif", -14, 0, 0, 0, FW_HEAVY,
		DEFAULT_PITCH | FF_SWISS, FALSE, FALSE, FALSE, 1,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY );
	NameFont = new TFont( "MS Sans Serif", -10, 0, 0, 0, FW_NORMAL,
		DEFAULT_PITCH | FF_SWISS, FALSE, FALSE, FALSE, 1,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY );
	InfoFont = new TFont( "MS Sans Serif", -10, 0, 0, 0, FW_BOLD,
		DEFAULT_PITCH | FF_SWISS, FALSE, FALSE, FALSE, 1,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY );
}

//
TSimWindow::~TSimWindow( void )
{
	KillTimer(TIMER_NUM);      // Get rid of timer
	delete SimBitmap;

	sw_cs_bg->SetButtonState( TButtonGadget::Up );
	sw_be_bg->SetButtonState( TButtonGadget::Up );

	*dltd = TRUE;
}

// Do any HWindow required setup tasks here, not in constructor
//
void	TSimWindow::SetupWindow( void )
{
	int i, num=4, rpt=0, num_hold1, num_hold2, num_hold3;

	do {
		SimBus->SearchROM(); num_hold1 = SimBus->num_parts;
		SimBus->SearchROM(); num_hold2 = SimBus->num_parts;
		SimBus->SearchROM(); num_hold3 = SimBus->num_parts;
		} while ((	(num_hold1!=num_hold2) ||
						(num_hold1!=num_hold3) ||
						(num_hold2!=num_hold3) )	&& rpt++<5 );

	SimBus->RecallEE();

	if ( SimBus->num_parts < 4 )
		num = SimBus->num_parts;

	for ( i=0 ; i<num ; i++ )
		{
		rpt=0;
		do
			SimBus->DS1820[i]->ReadSP();
		while ( !(SimBus->DS1820[i]->ValidSPData()) && rpt++<5 );

		THhold[i] = (int) SimBus->DS1820[i]->SPData[2];
		if ( THhold[i] >= 128 )
			THhold[i] -= 256;

		TLhold[i] = (int) SimBus->DS1820[i]->SPData[3];
		if ( TLhold[i] >= 128 )
			TLhold[i] -= 256;
		}

	// set the timer - get called every half-second
	if (!SetTimer(TIMER_NUM, 500U, 0))
		MessageBox( "Out of Timers", GetApplication()->GetName(),
			MB_ICONEXCLAMATION|MB_OK );

	TFrameWindow::SetupWindow();
}

// Paint the background and then the parts' data
//
void	TSimWindow::Paint(TDC& dc, BOOL, TRect&)
{
	TMemoryDC memDC(dc);
	int i, num=4;
	char hold[50];
	float temperature;

	memDC.SelectObject(*NewSimBitmap);

	memDC.SetTextAlign( TA_CENTER );

	if ( SimBus->num_parts < 4 )
		num = SimBus->num_parts;

	for ( i=0 ; i<num ; i++ )
		{
		PrintData( memDC, _TEMPNAME_, "TEMP", i );
		temperature = SimBus->DS1820[i]->ReadTemperature();
		sprintf( hold, "  %.1fC  ", temperature );
		PrintData( memDC, _TEMPVAL_, hold, i );

		if ( temperature >= (float) THhold[i] )
			hold[0]='Y';
		else
			hold[0]='N';
		PrintData( memDC, _ALARMTH_, hold, i );
		sprintf( hold+1, "%dC", THhold[i] );
		PrintData( memDC, _TH_, hold, i );

		if ( temperature <= (float) TLhold[i] )
			hold[0]='Y';
		else
			hold[0]='N';
		PrintData( memDC, _ALARMTL_, hold, i );
		sprintf( hold+1, "%dC", TLhold[i] );
		PrintData( memDC, _TL_, hold, i );

		PrintData( memDC, _NAME_, SimBus->DS1820[i]->ROMName, i );

		SimBus->DS1820[i]->ROMString( hold );
		PrintData( memDC, _CODE_, hold, i );
		}

	for ( ; i<4 ; i++ )
		{
		PrintData( memDC, _TEMPNAME_, "", i );
		PrintData( memDC, _TEMPVAL_, "..", i );
		PrintData( memDC, _ALARMTH_, "N", i );
		PrintData( memDC, _TH_, "N..", i );
		PrintData( memDC, _ALARMTL_, "N", i );
		PrintData( memDC, _TL_, "N..", i );
		PrintData( memDC, _NAME_, "No part found", i );
		PrintData( memDC, _CODE_, "..", i );
		}

	dc.BitBlt(0, 0, NewSimBitmap->Width(), NewSimBitmap->Height(),
		memDC, 0, 0, SRCCOPY);
}

//
void	TSimWindow::PaintData(TDC& dc)
{
	int i, num=4;
	char hold[20];
	float temperature;

	if ( SimBus->num_parts < 4 )
		num = SimBus->num_parts;

	for ( i=0 ; i<num ; i++ )
		{
		PrintData( dc, _TEMPNAME_, "TEMP", i );
		temperature = SimBus->DS1820[i]->ReadTemperature();
		sprintf( hold, "  %.1fC  ", temperature );
		PrintData( dc, _TEMPVAL_, hold, i );

		if ( temperature >= (float) THhold[i] )
			hold[0]='Y';
		else
			hold[0]='N';
		PrintData( dc, _ALARMTH_, hold, i );

		if ( temperature <= (float) TLhold[i] )
			hold[0]='Y';
		else
			hold[0]='N';
		PrintData( dc, _ALARMTL_, hold, i );
		}
}

void	TSimWindow::PrintData( TDC& dc, int whichReg, char* data, int partNum )
{
	TBrush* fill_brush;
	TMemoryDC memDC( dc );

	switch ( whichReg )
		{
		case _TEMPNAME_:
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::LtRed );
			dc.SetTextColor( TColor::LtYellow );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum] + TEMPDELTA, "TEMP" );
			break;

		case _TEMPVAL_:
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::LtRed );
			dc.SetTextColor( TColor::LtYellow );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum] + TEMPDELTA + TDELTA, data );
			break;

		case _ALARMTH_:
			memDC.SelectObject( *AlarmBitmap );
			if ( data[0]=='N' )
				dc.BitBlt( SimROMx[partNum] - 28, SimROMy[partNum],
					AlarmBitmap->Width(), AlarmBitmap->Height(),
					memDC, 0, 0, WHITENESS );
			else
				dc.BitBlt( SimROMx[partNum] - 28, SimROMy[partNum],
					AlarmBitmap->Width(), AlarmBitmap->Height(),
					memDC, 0, 0, SRCCOPY );
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::White );
			dc.SetTextColor( TColor::LtRed );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum], "TH" );
			break;

		case _TH_:
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::White );
			dc.SetTextColor( TColor::LtRed );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum] + TDELTA, data+1 );
			break;

		case _ALARMTL_:
			memDC.SelectObject( *AlarmBitmap );
			if ( data[0]=='N' )
				dc.BitBlt( SimROMx[partNum] - 28, SimROMy[partNum] + TLDELTA,
					AlarmBitmap->Width(), AlarmBitmap->Height(),
					memDC, 0, 0, WHITENESS );
			else
				dc.BitBlt( SimROMx[partNum] - 28, SimROMy[partNum] + TLDELTA,
					AlarmBitmap->Width(), AlarmBitmap->Height(),
					memDC, 0, 0, SRCCOPY );
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::White );
			dc.SetTextColor( TColor::LtBlue );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum] + TLDELTA, "TL" );
			break;

		case _TL_:
			dc.SelectObject(*TempFont);
			dc.SetBkColor( TColor::White );
			dc.SetTextColor( TColor::LtBlue );
			dc.TextOut( SimROMx[partNum],
				SimROMy[partNum] + TLDELTA + TDELTA, data+1 );
			break;

		case _CODE_:
			dc.SelectObject(*InfoFont);
			dc.SetBkColor( TColor::LtGray );
			dc.SetTextColor( TColor::Black );
			if ( partNum==0 || partNum==2 )
				dc.TextOut( SimROMx[partNum],
					SimROMy[partNum] + CODEDELTAPOS, data );
			else
				dc.TextOut( SimROMx[partNum],
					SimROMy[partNum] + CODEDELTANEG, data );
			break;

		case _NAME_:
			fill_brush = new TBrush( TColor::LtGray );

			dc.SelectObject(*NameFont);
			dc.SetBkColor( TColor::LtGray );
			dc.SetTextColor( TColor::Black );
			if ( partNum==0 || partNum==2 )
				{
				dc.FillRect( SimROMx[partNum] - 64,
								 SimROMy[partNum] + NAMEDELTAPOS - 2,
								 SimROMx[partNum] + 64,
								 SimROMy[partNum] + NAMEDELTAPOS + 16,
								 *fill_brush );
				dc.TextOut( SimROMx[partNum],
					SimROMy[partNum] + NAMEDELTAPOS, data );
				}
			else
				{
				dc.FillRect( SimROMx[partNum] - 60,
								 SimROMy[partNum] + NAMEDELTANEG - 2,
								 SimROMx[partNum] + 60,
								 SimROMy[partNum] + NAMEDELTANEG + 16,
								 *fill_brush );
				dc.TextOut( SimROMx[partNum],
					SimROMy[partNum] + NAMEDELTANEG, data );
				}
			delete fill_brush;
			break;
		}
}

void	TSimWindow::EvTimer( UINT )
{
	TClientDC dc(*this);
	int i, num=4, rpt;

	SimBus->AlarmSearch();

	if ( SimBus->num_parts < 4 )
		num = SimBus->num_parts;

	for ( i=0 ; i<num ; i++ )
		{
		rpt = 0;
		do
			SimBus->DS1820[i]->ReadSP();
			while ( !(SimBus->DS1820[i]->ValidSPData()) && rpt++<5 );
		}

	SimBus->ConvertT();

	dc.SetTextAlign( TA_CENTER );

	PaintData( dc );
}

void	TSimWindow::EvLButtonDown( UINT, TPoint& point )
{
	int ROMRegion;
	char in_name[NAME_LEN];

	if ( point.x < ( SimROMx[0]+ SimROMx[1] )/2 )
		ROMRegion = 0;
	else
		{
		if ( point.x < ( SimROMx[1]+ SimROMx[2] )/2 )
			ROMRegion = 1;
		else
			{
			if ( point.x < ( SimROMx[2]+ SimROMx[3] )/2 )
				ROMRegion = 2;
			else
				ROMRegion = 3;
			}
		}

	if ( ROMRegion < SimBus->num_parts )
		{
		SimBus->ROMNames
			->FindROMName( SimBus->DS1820[ROMRegion]->ROMCode, in_name );
		if ( TInNameDialog( this, IN_DLG, SimBus, ROMRegion, in_name )
				.Execute() == IDOK )
			{
			SimBus->ROMNames->AddROMName( SimBus->DS1820[ROMRegion]->ROMCode,
				in_name );
			SimBus->DS1820[ROMRegion]->SetName( in_name );
			Invalidate();
			SetupDirty = 1;
			}
		}
}
