#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <conio.h>
#include "thermlib.h"

void	ThermLib::InitThermLib( void )
{
	int i;

	dataPortA = 0xFF;
	dataPortC = 0x14; // For certain machines (namely the IBM ThinkPad),
							// the outport port is disabled when bit 5 of the
							// parallel port I/O byte is set to 1.  Therefore,
							// set that bit to 0.

	status = 0;
	SetCode();

	for ( i=0 ; i<9 ; i++ )
		SPData[i] = 0;

	SysClock = (ulong far*) ((void __seg *)(0x40) + (void __near *)(0x6C));

	FindPort();

	outportb( portLPT[numLPT-1],		dataPortA );
	outportb( portLPT[numLPT-1]+2,	dataPortC );
}

void	ThermLib::FindPort( void )
{
	int rpt;
	void __seg* portSeg = (void __seg*)(0x40);

	portLPT[0] = *( (int far*) ( portSeg + (void __near*)(0x08) ) );
	portLPT[1] = *( (int far*) ( portSeg + (void __near*)(0x0A) ) );
	portLPT[2] = *( (int far*) ( portSeg + (void __near*)(0x0C) ) );
	portLPT[3] = *( (int far*) ( portSeg + (void __near*)(0x0E) ) );

	if ( numLPT==0 )
		{
		if ( _argc > 1 )
			switch ( _argv[1][0] )
				{
				case '1':	numLPT = 1;
								break;
				case '2':	numLPT = 2;
								break;
				case '3':	numLPT = 3;
								break;
				case '4':	numLPT = 4;
								break;
				}
		else
			{
			numLPT = 1;
			if ( portLPT[numLPT-1]!=0 )
				{
				rpt = 0;
				while ( rpt++ < 5 )
					if ( OneWireReset() != -1 )
						return;
				}

			numLPT = 2;
			if ( portLPT[numLPT-1]!=0 )
				{
				rpt = 0;
				while ( rpt++ < 5 )
					if ( OneWireReset() != -1 )
						return;
				}

			numLPT = 3;
			if ( portLPT[numLPT-1]!=0 )
				{
				rpt = 0;
				while ( rpt++ < 5 )
					if ( OneWireReset() != -1 )
						return;
				}

			numLPT = 4;
			if ( portLPT[numLPT-1]!=0 )
				{
				rpt = 0;
				while ( rpt++ < 5 )
					if ( OneWireReset() != -1 )
						return;
				}

			if ( portLPT[0] != 0 )
				numLPT = 1;
			else
				{
				if ( portLPT[1] != 0 )
					numLPT = 2;
				else
					{
					if ( portLPT[2] != 0 )
						numLPT = 3;
					else
						if ( portLPT[3] != 0 )
							numLPT = 4;
					}
				}
			}
		}
}

int	ThermLib::WrSP( uchar TH, uchar TL )
{
	if ( OneWireWriteByte( COM_WR_SP ) == COM_WR_SP )
		if ( OneWireWriteByte( TH ) == TH )
			if ( OneWireWriteByte( TL ) == TL )
				return 0;

	return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::RdSP( uchar *data )
{
	int i, hold;

	CRCValue=0;
	if ( OneWireWriteByte( COM_RD_SP ) == COM_RD_SP )
		{
		for ( i=0 ; i<8 ; i++ )
			{
			hold = OneWireWriteByte( 0xFF );
			if ( hold != -1 )
				CalcCRC( &CRCValue, data[i] = (uchar) hold );
			else
				return STAT_NO_DEVICE_FOUND;
			}

		hold = OneWireWriteByte( 0xFF );
		if ( hold != -1 )
			{
			if ( CRCValue==(data[i] = (uchar) hold) )
				return 0;
			else
				return STAT_CRC_INVALID;
			}
		}

	return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::CnvtT( void )
{
	if ( OneWireWriteByte( COM_CNVT_T ) == COM_CNVT_T )
		return 0;
//		switch ( OneWireWriteBit( 1 ) )
//			{
//			case 1:	return 0;
//			case 0:	return STAT_THERM_BUSY;
//			}

	return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::CpSP( void )
{
	if ( OneWireWriteByte( COM_CP_SP ) == COM_CP_SP )
		return 0;
	else
		return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::RcEE( void )
{
	if ( OneWireWriteByte( COM_RC_EE ) == COM_RC_EE )
		switch ( OneWireWriteBit( 1 ) )
			{
			case 1:	return 0;
			case 0:	return STAT_THERM_BUSY;
			}

	return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::RdPS( void )
{
	if ( OneWireWriteByte( COM_RD_PS ) == COM_RD_PS )
		switch ( OneWireWriteBit( 1 ) )
			{
			case 1:	return 0;
			case 0:	return STAT_PARASITE_POWER;
			}

	return STAT_NO_DEVICE_FOUND;
}

int	ThermLib::TimeOut( int TCmd )
{
	// Save value from system clock
	if (TCmd != CHECKTIME)
		if ( _argc < 3 )
			InitTime = *SysClock + TCmd;
		else
			InitTime = *SysClock + _argv[2][0] - '2';

	if ( _argc >= 3 && _argv[2][0]=='n' )
		return FALSE;

	if (*SysClock > InitTime)
		return TRUE;
	else
		return FALSE;
}

int	ThermLib::ParallelCom( uchar type )
{
	int DOWRes;

	// Set timeout
	TimeOut( INIT1WTIME );

	// Set up port
	outportb( portLPT[numLPT-1], dataPortA=type );

	// Drive 14 (chip select) low
	outportb( portLPT[numLPT-1]+2, SET_ENIB_LOW );

	// Wait for DS1480 to issue busy signal
	while (( (inportb( portLPT[numLPT-1]+1 )^PORT_O1_BSY1B) & PORT_O1_OR_O2 )
				&& !TimeOut( CHECKTIME ) );

	// Now change the state of the data lines to all high
	outportb( portLPT[numLPT-1], dataPortA=0xFF );

	// Wait for busy signal to go away
	while (!( (inportb( portLPT[numLPT-1]+1 )^PORT_O1_BSY1B) & PORT_O1_OR_O2 )
				&& !TimeOut(CHECKTIME));

	// Drive clk line low to get bit
	outportb( portLPT[numLPT-1], SET_D_CLKB_LOW );

	// Burn a few cycles
	TimeOut( CHECKTIME );

	// Get bit result
	if ( (inportb( portLPT[numLPT-1]+1 )^PORT_O1_BSY1B) & PORT_O1_OR_O2 )
		DOWRes = 1;
	else DOWRes = 0;

	// Drive pin 14 back high again
	outportb( portLPT[numLPT-1]+2, SET_ENIB_HIGH );

	// Return 1-wire result
	if ( !TimeOut(CHECKTIME) )
		return DOWRes;
	else
		return -1;
}

int	ThermLib::OneWireReset( void )
{
	return ParallelCom( PORT_RESET_DATA );
}

int	ThermLib::OneWireWriteBit( int data )
{
	if ( data==0 )
		return ParallelCom( PORT_WRITE0_DATA );
	else
		return ParallelCom( PORT_WRITE1_DATA );
}

int	ThermLib::OneWireWriteByte( int data )
{
	int i=0, j, hold=0, send;

	do {
		send = data & 0x01;
		j = OneWireWriteBit( send );
		hold |= j<<i;
		data >>= 1;
		}
	while ( ( ++i<8 ) && ( j!=-1 ) );

	if ( j!=-1 )
		return hold;
	else
		return -1;
}

void	ThermLib::CalcCRC( uchar *CRCVal, uchar value )
{
	uchar odd, bcnt;

	for ( bcnt=0 ; bcnt<8 ; bcnt++ )
		{
		odd = ( value ^ *CRCVal ) & 0x01;
		*CRCVal >>= 1;
		value >>= 1;
		if ( odd != 0 ) *CRCVal ^= 0x8C;
		}
}

int	ThermLib::ValidCRC( uchar code_len, uchar *code )
{
	int i;
	uchar CRCVal=0;

	for ( i=0 ; i<(code_len-1) ; i++ )
		CalcCRC( &CRCVal, code[i] );

	if ( *(code+i) != CRCVal )
		return FALSE;
	else
		return TRUE;
}

void	ThermLib::ROMString( char *code_string )
{
	int i;
	char str_temp[10];

	*code_string = '\0';

	for ( i=0 ; i<8 ; i++ )
		{
		sprintf( str_temp, "%02X", (int) ROMCode[7-i] ); // '7-' added v2.0
		strcat( code_string, str_temp );
		}
}

void	ThermLib::SetCode( uchar *code )
{
	int i;

	for( i=0 ; i<8 ; i++ )
		ROMCode[i]=code[i];
}

void	ThermLib::SetCode( void )
{
	int i;

	for( i=0 ; i<8 ; i++ )
		ROMCode[i]=0;
}

int	ThermLib::SameCode( uchar *code )
{
	int i;

	for( i=0 ; i<8 ; i++ )
		if ( ROMCode[i]!=code[i] )
			return FALSE;

	return TRUE;
}

int	ThermLib::ReadStatus( int mask )
{
	int holdStatus = status;
	status &= ~mask;
	return holdStatus;
}

void	ThermLib::SetStatus( int flags )
{
	status |= flags;
}

void	ThermLib::ClearStatus( int mask )
{
	status &= ~mask;
}

float	ThermLib::ReadTemperature( void )
{
	int temp_int;
	float frac_deg;

	temp_int = SPData[0]/2;

	if ( SPData[1] == 0xFF )
		temp_int -= 128;

	if ( SPData[7] != 0 )
		frac_deg =(SPData[7]-SPData[6])/((float)SPData[7]);
	else
		frac_deg = 0.25;

	return (((float)temp_int - 0.25) + frac_deg);
}

ThermPart::ThermPart( int portNum )
{
	AlarmState[0] = FALSE;
	AlarmState[1] = FALSE;
	AlarmState[2] = FALSE;
	AlarmCond = FALSE;
	SetName( "No name" );

	numLPT = portNum;
	InitThermLib();
}

int	ThermPart::MatchROM( void )
{
	int i;

	if ( OneWireReset() == 0 )
		if ( OneWireWriteByte( COM_MATCH_ROM ) == COM_MATCH_ROM )
			for ( i=0 ; i<8 ; i++ )
				if ( OneWireWriteByte( ROMCode[i] ) != ROMCode[i] )
					return STAT_NO_DEVICE_FOUND;

	if ( i==8 )
		return 0;
	else
		return STAT_NO_DEVICE_FOUND;
}

int	ThermPart::EvalAlarmCond( void )
{
	if ( AlarmState[0] && AlarmState[1] && AlarmState[2] )
		AlarmCond = TRUE;
	else if ( ~( AlarmState[0] || AlarmState[1] || AlarmState[2] ) )
		AlarmCond = FALSE;
	return AlarmCond;
}

void	ThermPart::AddAlarm( int cond )
{
	AlarmState[2] = AlarmState[1];
	AlarmState[1] = AlarmState[0];
	AlarmState[0] = cond;
}

ThermBus::ThermBus( int portNum )
{
	ROMNames = new ROMNameTable;

	numLPT = portNum;

	InitThermLib();

	num_parts = 0;
	for ( int i=0 ; i<MAX_PARTS ; i++ )
		DS1820[i] = new ThermPart( portNum );
}

ThermBus::~ThermBus( void )
{
	delete[] DS1820;
}

void	ThermBus::SetPort( int portNum )
{
	numLPT = portNum;

	for ( int i=0 ; i<MAX_PARTS ; i++ )
		DS1820[i]->numLPT = portNum;
}

void	ThermBus::ReadROM( void )
{
	int i, hold;

	CRCValue=0;

	if ( OneWireReset() == 0 )
		if ( OneWireWriteByte( COM_READ_ROM ) == COM_READ_ROM )
			{
			for ( i=0 ; i<7 ; i++ )
				if ( (hold=OneWireWriteByte( 0xFF )) != -1 )
					CalcCRC( &CRCValue, ROMCode[i] = (uchar) hold );
				else
					{
					SetStatus( STAT_NO_DEVICE_FOUND );
					return;
					}
			if ( (hold=OneWireWriteByte( 0xFF )) != -1 )
				{
				if ( CRCValue!=( ROMCode[i]=(uchar) hold ) )
					SetStatus( STAT_CRC_INVALID );
				return;
				}
			}

	SetStatus( STAT_NO_DEVICE_FOUND );
}

// SearchBit performs the two reads during a ROM or Alarm Search
// It returns 3 (no devices on bus), 2 (only 1's on bus),
// 1 (only 0's on bus), or 0 (both 0's and 1's on bus)

int	ThermBus::SearchBit( void )
{
	int hold;

	if ( (hold=OneWireWriteBit( 1 )<<1) == -1 )
		return -1;

	if ( (hold|=OneWireWriteBit( 1 )) == -1 )
		return -1;

	return hold;
}

void	ThermBus::SetROMBit( uchar *CurROM, int bitPos, int value )
{
	if ( value )
		*(CurROM + (bitPos-1)/8) |= (1<<((bitPos-1)%8));
	else
		*(CurROM + (bitPos-1)/8) &= ~(1<<((bitPos-1)%8));
}

// For functionality of FindROM procedure, see p. 54 of
// Touch Memory Standards book

int	ThermBus::FindROM( int busCom, uchar *CurROM,
	int *lastDiscrep, int *done, int num )
{
	int sb, discrep, val, bitPos, i;

	if ( num==0 )
		{
		*done = FALSE;
		*lastDiscrep = 0;
		for ( i=0 ; i<8 ; i++ )
			CurROM[i] = 0;
		}

/*	if ( *done == TRUE )
		{
		*done = FALSE;
		return 0;
		}
*/

	if ( OneWireReset()!=0 ) // No presence pulse
		{
		*lastDiscrep = 0;
		return -1;
		}

	discrep = 0;

	if ( OneWireWriteByte( busCom ) != busCom )
		return -1;

	for ( bitPos=1 ; bitPos<=64 ; bitPos++ )
		{
		sb = SearchBit();
		val = *(CurROM + (bitPos-1)/8) & (1<<((bitPos-1) % 8));
		if ( sb==-1 ) // Bad searchbit result
			return -1;
		if ( sb==3 ) // No more devices on bus
			{
			*lastDiscrep = 0;
			return 0;
			}
		else
			{
			if ( sb==0 )
				{
				if ( bitPos==*lastDiscrep )
					SetROMBit( CurROM, bitPos, 1 );
				else // if !(bitPos==*lastDiscrep)
					{
					if ( bitPos>*lastDiscrep )
						{
						SetROMBit( CurROM, bitPos, 0 );
						discrep = bitPos;
						}
					else // If !(bitPos>*lastDiscrep)
						if ( val==0 )
							discrep = bitPos;
					}
				}
			else // If !(sb==0)
				SetROMBit( CurROM, bitPos, sb & ~0x01 );
			val = *(CurROM + (bitPos-1)/8) & (1<<((bitPos-1) % 8));
			val >>= (bitPos-1) % 8;
			if ( OneWireWriteBit( val ) != val )
				return -1;
			}
		}

	*lastDiscrep = discrep;

	if ( discrep==0 )
		*done = 1;

	return 1;
}

void	ThermBus::SearchROM( void )
{
	uchar hc[8];
	int ld=0, num=0, done=FALSE;
	char name[NAME_LEN];

	while ( !done && num<MAX_PARTS &&
			( FindROM( COM_SEARCH_ROM, hc, &ld, &done, num ) )==1 )
		{
		DS1820[num]->SetCode( hc );
		ROMNames->FindROMName( hc, name );
		DS1820[num]->SetName( name );
		if ( !ValidCRC( 8, hc ) )
			DS1820[num]->SetStatus( STAT_CRC_INVALID );
		DS1820[num++]->ClearStatus( STAT_NO_DEVICE_FOUND );
		}

	num_parts = num;

	while ( num<MAX_PARTS )
		{
		DS1820[num]->SetCode();
		DS1820[num]->SetName( "No device found" );
		DS1820[num++]->SetStatus( STAT_NO_DEVICE_FOUND );
		}

	if ( num==0 )
		SetStatus( STAT_NO_DEVICE_FOUND );

}

void	ThermBus::AlarmSearch( void )
{
	uchar hc[8];
	int ld=0, num=0, done=FALSE, i;
	int changedState=0, find_ret;

	while ( !done && num<num_parts &&
			( find_ret=FindROM( COM_ALARM_SEARCH, hc, &ld, &done, num++ ) )==1 )
		for ( i=0 ; i<num_parts ; i++ )
			if ( DS1820[i]->SameCode( hc ) )
				{
				DS1820[i]->AddAlarm( TRUE );
				changedState |= 1<<i;
				}

	if ( find_ret==-1 )
		return;

	for ( i=0 ; i<num_parts ; i++ )
		if ( !( changedState & (1<<i)) )
			DS1820[i]->AddAlarm( FALSE );
}

int	ThermBus::SkipROM( void )
{
	if ( OneWireReset()==0 )
		if ( OneWireWriteByte( COM_SKIP_ROM ) == COM_SKIP_ROM )
			return 0;

	return STAT_NO_DEVICE_FOUND;
}

ROMNameTable::ROMNameTable( void )
{
	int i;

	num_names = 0;

	ROMCodeList = new uchar* [MAX_NAMES];

	ROMNameList = new char* [MAX_NAMES];

	for ( i=0 ; i<MAX_NAMES ; i++ )
		{
		ROMCodeList[i] = new uchar [8];
		ROMNameList[i] = new char [NAME_LEN];
		}
}

ROMNameTable::~ROMNameTable( void )
{
	int i;

	for ( i=0 ; i<MAX_NAMES ; i++ )
		{
		delete[] ROMCodeList[i];
		delete[] ROMNameList[i];
		}

	delete[] ROMCodeList;
	delete[] ROMNameList;
}

int	ROMNameTable::SameCode( uchar *code1, uchar *code2 )
{
	int i;

	for( i=0 ; i<8 ; i++ )
		if ( code1[i]!=code2[i] )
			return FALSE;

	return TRUE;
}

void	ROMNameTable::AddROMName( uchar *code, char *name )
{
	int i=0;

	while ( i<num_names && !SameCode( code, ROMCodeList[i] ) ) i++;

	if ( i<MAX_NAMES )
		{
		strncpy( ROMNameList[i], name, NAME_LEN );

		if ( i==num_names )
			{
			for ( i=0 ; i<8 ; i++ )
				ROMCodeList[num_names][i] = code[i];
			num_names++;
			}
		}
}

void	ROMNameTable::FindROMName( uchar *code, char *name )
{
	int i=0;

	while ( i<num_names && !SameCode( code, ROMCodeList[i] ) ) i++;

	if ( i<num_names )
		strncpy( name, ROMNameList[i], NAME_LEN );
	else
		strncpy( name, "No name found", NAME_LEN );
}
