/*************************************
*
* Revision History
*
*   Written by Jim Douglass
*
*   Rev. 1.0 :  Released 3/29/95
*   Rev. 1.1 :  Fixed two problems:
*                 Program did not work on IBM ThinkPad laptop because
*                   of parallel port output shut-down bit in I/O
*                   byte.  Bit 5 of I/O register (portLPT+2) must be
*                   0 to allow output byte to function.  See thermlib.cpp,
*                   ThermLib::InitThermLib for changes.
*                 Fixed problem with command-line option for specifying
*                   which LPT port to use.  Also increased default length
*                   of system clock time-out.  Added second option to specify
*                   system clock delay before rejection of 1-wire Reset,
*                   Read, or Write.  See thermlib.cpp, ThermLib::FindPort
*                   and ThermLib::TimeOut for changes.  Additional
*                   (undocumented) option:  If second parameter = '0',
*                   force every communication to fail.  If '1', fail after
*                   1 tick in the system clock, and so forth.  If 'n',
*                   force no time-out based on system clock.
*               Released 3/31/95
*   Rev. 2.0 :  ROM Codes were displayed in reverse of the Touch convention
*                 (family code last; see thermlib.cpp, ThermLib::ROMString
*                 for changes).
*               Added capability of saving port number and part name
*                 information to a file.
*               Added capability of selecting the desired parallel port.
*
*************************************/

#include "win1820.h"

int SetupDirty;

char HelpDS1820Overview[] =
"The DS1820 is a Digital Thermometer from Dallas Semiconductor that has the \
following features:\n\
  o  A patented 1-Wire data interface\n\
  o  Multidrop capability\n\
  o  Requires no external components\n\
  o  Can be powered from data line\n\
  o  Zero standby power\n\
  o  Measures temperature from -55C to 125C\n\
  o  Measures temperature in 200ms (typ.)\n\
  o  User-definable, nonvolatile temperature\n\
\talarm settings\n\
  o  Alarm Search command identifies devices\n\
\toutside limits\n\n\
See the DS1820 data sheet for details.";

char HelpDS1820ROMCommands[] =
"Following is a list of the 1-Wire ROM protocols:\n\n\
  o  Read ROM (33h)\n\
  o  Match ROM (55h)\n\
  o  Skip ROM (CCh)\n\
  o  Search ROM (F0h)\n\
  o  Alarm Search (ECh)\n\n\
See the DS1820 data sheet for details.";

char HelpDS1820CommandSet[] =
"Following is a list of the DS1820 Temperature and Memory protocols:\n\n\
  o  Convert Temperature (44h)\n\
  o  Read Scratchpad (BEh)\n\
  o  Write Scratchpad (4Eh)\n\
  o  Copy Scratchpad (48h)\n\
  o  Recall EEPROM (B8h)\n\
  o  Read Power Supply (B4h)\n\n\
See the DS1820 data sheet for details.";

char HelpSimOverview[] =
"The Control System Simulation screen demonstrates how a controller might \
run a bus of DS1820s. The data from up to 4 Digital Thermometers is \
displayed on the screen.\n\n\
When the screen is brought up, a Search ROM is performed to determine \
what parts are on the bus. Next, every half-second a Convert Temperature \
protocol is sent to and the scratchpad read from each part on the bus.\n\n\
Note that the Control System Simulation and the Bus Experimentation \
windows cannot be opened at the same time, in order to prevent bus \
contention between the two.";

char HelpSimDisplay[] =
"The thermostat settings, the ROM code, and the name are displayed for \
each part. The temperature of each part is displayed with 0.1C \
resolution.\n\n\
If a part's temperature is at or above the value in its TH register, an \
alarm symbol will appear around its TH display. Likewise, if a part's \
temperature is at or above the value in its TL register, an alarm symbol \
will appear around its TL display.";

char HelpSimName[] =
"A part's name can be changed by clicking the left mouse button in a \
part's display region. A dialog box will appear that displays a part's ROM \
code, its old name, and an edit box for entering the new name. After the \
desired name has been entered, the user can press 'OK' to accept the change, \
or 'CANCEL' to ignore the change.";

char HelpSimClose[] =
"The Control System Simulation will continue to run until the window is \
closed either by selecting the Close system menu item or by pressing Alt-F4.";

char HelpBusOverview[] =
"The Bus Experimentation window allows the user to exercise the bus as per \
the DS1820 data sheet. All of the ROM functions, temperature functions, and \
memory functions can be performed on any individual part on the bus or on \
every part on the bus together.\n\n\
The window consists of pushbuttons that initiate various commands, radio \
buttons that choose general settings, a list box of all of the parts found \
on the bus, and a display of various data. The screen is updated whenever \
the user initiates an action.";

char HelpBusDisplay[] =
"The scratchpad values read from the selected part, its ROM code, and \
its name are displayed. The temperature of the selected part is also \
displayed, with 0.01C resolution.\n\n\
If the Skip ROM radio button is selected, all of the displayed values are \
from reads of all of the parts on the bus in parallel. That results in data \
that is a 'wired-AND' of all of the parts' data. Consequently, the Skip ROM \
will only result in valid data if there is only one part on the bus. See the \
DS1820 data sheet for details.";

char HelpBusListBox[] =
"The ROM list box contains each part's ROM code and its name. If an Alarm \
Search is performed, all of the parts that are in an Alarm Condition will \
have an asterisk (*) before their listings.\n\n\
To select a particular part on the bus, click the left mouse button on the \
appropriate line in the ROM list box. If the Match ROM radio button is \
selected, subsequent protocols will be sent to the selected part.";

char HelpBusName[] =
"A part's name can be changed by double-clicking the left mouse button on a \
part's data line in the ROM list box. A dialog box will appear that displays \
the part's ROM code, its old name, and an edit box for entering the new name. \
After the desired name has been entered, the user can press 'OK' to accept \
the change, or 'CANCEL' to ignore the change.";

char HelpBusWriteData[] =
"Near the center of the Bus Experimentation window are two buffers for Write \
Scratchpad values. Before sending a Write SP protocol, the user must enter \
valid data into these edit boxes. That can be done by putting the cursor in \
the appropriate edit box and then typing in the desired values. Data can be \
entered in the following formats:\n\
\tDec:\t[-]XXX\t\tX=[0-9]\n\
\tHex:\tXXh\t\tX=[0-9,A-F]\n\
\tBin:\tXXXXXXXXb\tX=[0-1]\n\
The data scanner is not case-sensitive, and leading 0's are not necessary. \
Valid data ranges from decimal 0 to 255 for hex and bin values, and from \
decimal -128 to 127 for decimal values.";

char HelpBusSetup[] =
"At the bottom of the Bus Experimentation window are two sections called \
SETUP FILE and PARALLEL/PRINTER PORT. In SETUP FILE there is one button for \
loading setup data and one for saving setup data. Each pops up a dialog \
window that allows the user to browse for a path and name. Setup files are \
text files that contain information about the preferred parallel/printer \
port as well as ROM code-part name mapping.\n\n\
The PARALLEL/PRINTER PORT section contains check boxes for selecting the \
desired port. Only valid ports can be selected. Normally, these boxes need \
not be used; if the kit is attached to a port when starting the program, \
the correct port should be found automatically.";

char HelpBusClose[] =
"The Bus Experimentation window can be closed either by selecting the Close \
system menu item or by pressing Alt-F4.";

DEFINE_RESPONSE_TABLE1( TInNameDialog, TDialog )
	EV_EN_KILLFOCUS( IN_NEW_NAME, UpdateName ),
END_RESPONSE_TABLE;

TInNameDialog
	::TInNameDialog( TWindow* parent, TResId resId, ThermBus* MainBus,
		int ROMRegion, char* in_text )
	:TDialog( parent, resId )
{
	hold_text = in_text;

	InNameBus = MainBus;
	InNameNum = ROMRegion;
}

void	TInNameDialog::UpdateName( void )
{
	GetDlgItemText( IN_NEW_NAME, hold_text, NAME_LEN );
}

void	TInNameDialog::SetupWindow( void )
{
	char temp[50];

	sprintf( temp, "Changing Name For ROM %d", InNameNum+1 );
	SetCaption( temp );

	InNameBus->DS1820[InNameNum]->ROMString( temp );
	SetDlgItemText( IN_ROM_CODE, temp );

	SetDlgItemText( IN_OLD_NAME, InNameBus->DS1820[InNameNum]->ROMName );
	SetDlgItemText( IN_NEW_NAME, InNameBus->DS1820[InNameNum]->ROMName );
}

DEFINE_RESPONSE_TABLE1( THelpDialog, TDialog )
	EV_WM_CLOSE,
	EV_WM_DESTROY,
	EV_LBN_SELCHANGE( HELP_LIST_BOX, DisplayHelp ),
	EV_COMMAND( HELP_DUMMY_BUTTON, DummyFunction ),
END_RESPONSE_TABLE;

THelpDialog::THelpDialog( TWindow* parent, TResId resId,
	TButtonGadget* hlp_bg, BOOL* del_hlp )
	:TDialog( parent, resId )
{
	dltd = del_hlp;
	*dltd = FALSE;

	hw_hlp_bg = hlp_bg;

	hw_hlp_bg->SetButtonState( TButtonGadget::Indeterminate );
}

void	THelpDialog::EvDestroy( void )
{
	*dltd = TRUE;

	hw_hlp_bg->SetButtonState( TButtonGadget::Up );

	TDialog::EvDestroy();
}

void	THelpDialog::SetupWindow( void )
{
	TDialog::SetupWindow();

	SendDlgItemMessage( HELP_LIST_BOX, LB_RESETCONTENT, 0, 0 );

	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "DS1820 INFORMATION" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Overview" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  ROM Protocols" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Temp and Memory Protocols" );

	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "CONTROL SYSTEM SIMULATION" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Overview" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Display Contents" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Renaming Parts" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Closing" );

	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "BUS EXPERIMENTATION" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Overview" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Display Contents" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  ROM List Box" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Renaming Parts" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Write Scratchpad Buffer" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Setup Files and Ports" );
	SendDlgItemMessage( HELP_LIST_BOX, LB_ADDSTRING, 0,
		(LPARAM) "  o  Closing" );

	SendDlgItemMessage( HELP_LIST_BOX, LB_SETCURSEL, 0, 0 );

	DisplayHelp();
}

void	THelpDialog::DisplayHelp( void )
{
	enum { DS1820Over1, DS1820Over2, DS1820ROMCommands, DS1820CommandSet,
			 SimOver1, SimOver2, SimDisplay, SimName, SimClose,
			 BusOver1, BusOver2, BusDisplay, BusListBox,
			 BusName, BusWriteData, BusSetup, BusClose };

	switch ( (int) SendDlgItemMessage( HELP_LIST_BOX, LB_GETCURSEL ) )
		{
		case DS1820Over1:
		case DS1820Over2:
			SetDlgItemText( HELP_TEXT_BOX, HelpDS1820Overview );
			break;
		case DS1820ROMCommands:
			SetDlgItemText( HELP_TEXT_BOX, HelpDS1820ROMCommands );
			break;
		case DS1820CommandSet:
			SetDlgItemText( HELP_TEXT_BOX, HelpDS1820CommandSet );
			break;
		case SimOver1:
		case SimOver2:
			SetDlgItemText( HELP_TEXT_BOX, HelpSimOverview );
			break;
		case SimDisplay:
			SetDlgItemText( HELP_TEXT_BOX, HelpSimDisplay );
			break;
		case SimName:
			SetDlgItemText( HELP_TEXT_BOX, HelpSimName );
			break;
		case SimClose:
			SetDlgItemText( HELP_TEXT_BOX, HelpSimClose );
			break;
		case BusOver1:
		case BusOver2:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusOverview );
			break;
		case BusDisplay:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusDisplay );
			break;
		case BusListBox:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusListBox );
			break;
		case BusName:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusName );
			break;
		case BusWriteData:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusWriteData );
			break;
		case BusSetup:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusSetup );
			break;
		case BusClose:
			SetDlgItemText( HELP_TEXT_BOX, HelpBusClose );
			break;
		}
}

void	THelpDialog::EvClose( void )
{
	*dltd = TRUE;

	hw_hlp_bg->SetButtonState( TButtonGadget::Up );

	TDialog::EvClose();
}

DEFINE_RESPONSE_TABLE1( TBusDialog, TDialog )
	EV_WM_CLOSE,
	EV_WM_DESTROY,
	EV_WM_PAINT,
	EV_COMMAND( EB_SKIPROM, SkipROM ),
	EV_COMMAND( EB_ALARMSEARCH, AlarmSearch ),
	EV_COMMAND( EB_SEARCHROM, SearchROM ),
	EV_COMMAND( EB_READROM, ReadROM ),
	EV_COMMAND( EB_MATCHROM, MatchROM ),
	EV_COMMAND( EB_CONVERTT, ConvertT ),
	EV_COMMAND( EB_READSP, ReadSP ),
	EV_COMMAND( EB_WRITESP, WriteSP ),
	EV_COMMAND( EB_COPYSP, CopySP ),
	EV_COMMAND( EB_RECALLEE, RecallEE ),
	EV_COMMAND( EB_READPS, ReadPS ),
	EV_LBN_SELCHANGE( EB_SEL_ROM_BOX, SelROM ),
	EV_LBN_DBLCLK( EB_SEL_ROM_BOX, UpdateROMName ),
	EV_COMMAND( EB_LOAD_SETUP, LoadSetup ),
	EV_COMMAND( EB_SAVE_SETUP, SaveSetup ),
	EV_COMMAND( EB_LPT1, SetPort1 ),
	EV_COMMAND( EB_LPT2, SetPort2 ),
	EV_COMMAND( EB_LPT3, SetPort3 ),
	EV_COMMAND( EB_LPT4, SetPort4 ),
END_RESPONSE_TABLE;

TBusDialog
	::TBusDialog( TWindow* parent, TResId resId, ThermBus* MainBus,
		TButtonGadget* cs_bg, TButtonGadget* be_bg, BOOL* del_be)
	:TDialog( parent, resId )
{
	ExpBus = MainBus;

	dltd = del_be;
	*dltd = FALSE;

	sw_cs_bg = cs_bg;
	sw_be_bg = be_bg;

	sw_cs_bg->SetButtonState( TButtonGadget::Indeterminate );
	sw_be_bg->SetButtonState( TButtonGadget::Indeterminate );

	TempFont = new TFont( "MS Sans Serif", -24, 0, 0, 0, FW_HEAVY,
		DEFAULT_PITCH | FF_SWISS, FALSE, FALSE, FALSE, 1,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY );
}

void	TBusDialog::SetupWindow( void )
{
	int tab[2]={ 6, 80 };
	char portText[20];

	TDialog::SetupWindow();

	WhichROM = 0;

	WriteTHData = 0;
	WriteTLData = 0;

	SearchROM();

	SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETTABSTOPS, 2, (LPARAM) tab );

	CreateROMList();

	SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETCURSEL, WhichROM, 0 );

	ExpBus->RecallEE();

	DoMatch = TRUE;
	CheckRadioButton( EB_MATCHROM, EB_SKIPROM, EB_MATCHROM );

	for ( int i=0 ; i<4 ; i++ )
		if ( ExpBus->portLPT[i] == 0 )
			{
			CheckDlgButton( EB_LPT1 + i, 2 );
			sprintf( portText, "LPT%d (none)", i+1 );
			SetDlgItemText( EB_LPT1 + i, portText );
			}
		else
			{
			CheckDlgButton( EB_LPT1 + i, 0 );
			sprintf( portText, "LPT%d (%3Xh)", i+1, ExpBus->portLPT[i] );
			SetDlgItemText( EB_LPT1 + i, portText );
			}

	CheckDlgButton( EB_LPT1 + ExpBus->numLPT - 1, 1 );

	UpdateScreen();
}

void	TBusDialog::EvClose( void )
{
	sw_cs_bg->SetButtonState( TButtonGadget::Up );
	sw_be_bg->SetButtonState( TButtonGadget::Up );

	*dltd = TRUE;

	TDialog::EvClose();
}

void	TBusDialog::EvDestroy( void )
{
	sw_cs_bg->SetButtonState( TButtonGadget::Up );
	sw_be_bg->SetButtonState( TButtonGadget::Up );

	*dltd = TRUE;

	TDialog::EvDestroy();
}

void	TBusDialog::AlarmSearch( void )
{
	ExpBus->AlarmSearch();
	ExpBus->AlarmSearch();
	ExpBus->AlarmSearch();
	ExpBus->AlarmSearch();

	CreateROMList();

	SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETCURSEL, WhichROM, 0 );
}

void	TBusDialog::SearchROM( void )
{
	int num_hold1, num_hold2, num_hold3, i=0, rpt;

	do {
		ExpBus->SearchROM(); num_hold1 = ExpBus->num_parts;
		ExpBus->SearchROM(); num_hold2 = ExpBus->num_parts;
		ExpBus->SearchROM(); num_hold3 = ExpBus->num_parts;
		} while ((	(num_hold1!=num_hold2) ||
						(num_hold1!=num_hold3) ||
						(num_hold2!=num_hold3) )	&& i++<5 );

	if ( i<5 )
		{
		for ( i=0 ; i<ExpBus->num_parts ; i++ )
			{
			rpt=0;
			do
				ExpBus->DS1820[i]->ReadSP();
			while ( !(ExpBus->DS1820[i]->ValidSPData()) && rpt++<5 );
			}
			
		CreateROMList();

		WhichROM = 0;
		SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETCURSEL, WhichROM, 0 );

		UpdateScreen();
		}
}

void	TBusDialog::ReadROM( void )
{
	ExpBus->ReadROM();
	UpdateScreen();
}

int	TBusDialog::CheckInData( char* data )
{
	int base, length, total, power=1, sign=0;
	char hold;

	length = strlen( data );

	while ( length > 0 && data[--length] == ' ');

	if ( length != 0 )
		{
		hold = data[length];
		switch ( hold )
			{
			case 'b':
			case 'B':	base = 2;
							break;
			case 'h':
			case 'H':	base = 16;
							break;
			default:		if ( hold < '0' || hold > '9' )
								return -1;
							else
								{
								base = 10;
								length++;
								}
							break;
			}
		total = 0;
		while ( --length >= 0 && ( hold = data[length] ) != ' ' )
			{
			switch ( base )
				{
				case 2:	if ( hold != '0' && hold != '1' )
								return -1;
							total += power * (hold - '0');
							break;
				case 10: if ( hold >= '0' && hold <= '9' )
								total += power * (hold - '0');
							else
								{
								if ( hold == '-' )
									sign = 1;
								else
									return -1;
								}
							break;
				case 16:	if ( hold >= '0' && hold <= '9' )
								total += power * (hold - '0');
							else
								if ( hold >= 'a' && hold <= 'f' )
									total += power * (hold - 'a' + 10);
								else
									if ( hold >= 'A' && hold <= 'F' )
										total += power * (hold - 'A' + 10);
									else
										return -1;
				}
			power *= base;
			}
		while ( length != -1 )
			if ( data[length--] != ' ' )
				return -1;
		if ( sign == 1 )
			{
			if ( total <= 128 )
				return ( 256 - total );
			else
				return -1;
			}
		if ( total >= 0 && total < 128 )
			return total;
		else
			if ( ( base == 2 || base == 16 ) && total < 256 )
				return total;
		}
	else
		if ( data[0]>='0' && data[0]<='9' )
			return ( data[0] - '0' );

	return -1;
}

void	TBusDialog::WriteSP( void )
{
	char in_data[20];
	int hold_val;

	GetDlgItemText( EB_WRITESPTH, in_data, 20 );

	if ( ( hold_val = CheckInData( in_data ) ) != -1 )
		WriteTHData = hold_val;
	else
		{
		MessageBox( "Invalid TH Value", "Error" );
		return;
		}

	GetDlgItemText( EB_WRITESPTL, in_data, 20 );

	if ( ( hold_val = CheckInData( in_data ) ) != -1 )
		WriteTLData = hold_val;
	else
		{
		MessageBox( "Invalid TL Value", "Error" );
		return;
		}

	DoCom( COM_WR_SP );
}

void	TBusDialog::SelROM( void )
{
	WhichROM = (int) SendDlgItemMessage( EB_SEL_ROM_BOX, LB_GETCURSEL );
	UpdateScreen();
}

void	TBusDialog::UpdateROMName( void )
{
	char in_name[NAME_LEN];

	ExpBus->ROMNames->FindROMName( ExpBus->DS1820[WhichROM]->ROMCode, in_name );

	if ( TInNameDialog( this, IN_DLG, ExpBus, WhichROM, in_name )
			.Execute() == IDOK )
		{
		ExpBus->ROMNames->AddROMName( ExpBus->DS1820[WhichROM]->ROMCode,
			in_name );
		ExpBus->DS1820[WhichROM]->SetName( in_name );
		CreateROMList();
		SetupDirty = 1;
		}

	UpdateScreen();

	SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETCURSEL, WhichROM, 0 );
}

void	TBusDialog::CreateROMList( void )
{
	int i;
	char list_data[60], temp1[30], temp2[30], alarmchar;

	SendDlgItemMessage( EB_SEL_ROM_BOX, LB_RESETCONTENT, 0, 0 );

	for ( i=0 ; i<ExpBus->num_parts ; i++ )
		{
		ExpBus->ROMNames->FindROMName( ExpBus->DS1820[i]->ROMCode, temp1 );
		if ( ExpBus->DS1820[i]->EvalAlarmCond() )
			alarmchar = '*';
		else
			alarmchar = ' ';
		ExpBus->DS1820[i]->ROMString( temp2 );
		sprintf( list_data, "%c\t%s\t%s", alarmchar, temp2, temp1 );
		SendDlgItemMessage( EB_SEL_ROM_BOX, LB_ADDSTRING, 0, (LPARAM)list_data );
		}
}

int	TBusDialog::DoCom( int com )
{
	if ( DoMatch )
		switch ( com )
			{
			case COM_CNVT_T:	ExpBus->DS1820[WhichROM]->ConvertT();
									break;
			case COM_RD_SP:	ExpBus->DS1820[WhichROM]->ReadSP();
									UpdateScreen();
									break;
			case COM_WR_SP:	ExpBus->DS1820[WhichROM]
										->WriteSP( WriteTHData, WriteTLData );
									break;
			case COM_CP_SP:	ExpBus->DS1820[WhichROM]->CopySP();
									break;
			case COM_RC_EE:	ExpBus->DS1820[WhichROM]->RecallEE();
									return ExpBus->DS1820[WhichROM]
										->ReadStatus( STAT_THERM_BUSY );
			case COM_RD_PS:	ExpBus->DS1820[WhichROM]->ReadPS();
									return ExpBus->DS1820[WhichROM]
										->ReadStatus( STAT_PARASITE_POWER );
			}
	else
		switch ( com )
			{
			case COM_CNVT_T:	ExpBus->ConvertT();
									break;
			case COM_RD_SP:	ExpBus->ReadSP();
									UpdateScreen();
									break;
			case COM_WR_SP:	ExpBus->WriteSP( WriteTHData, WriteTLData );
									break;
			case COM_CP_SP:	ExpBus->CopySP();
									break;
			case COM_RC_EE:	ExpBus->RecallEE();
									return ExpBus->ReadStatus( STAT_THERM_BUSY );
			case COM_RD_PS:	ExpBus->ReadPS();
									return ExpBus->ReadStatus( STAT_PARASITE_POWER );
			}
	return 0;
}

void	TBusDialog::EvPaint( void )
{
	TDialog::EvPaint();

	UpdateTemp();
}

void	TBusDialog::UpdateTemp( void )
{
	char temp[20];
	TClientDC dc(*this);
	TBrush fill_brush( TColor::Black );
	TRect winrect, fillrect;
	float winwidth=600.0, winheight=460.0;

	winrect = GetClientRect();
	fillrect.left = (int) (winrect.right*(385.0/winwidth));
	fillrect.top = (int) (winrect.bottom*(305.0/winheight));
	fillrect.right = (int) (winrect.right*(545.0/winwidth));
	fillrect.bottom = (int) (winrect.bottom*(345.0/winheight));
	sprintf( temp, "%6.2fC", temperature );
	dc.FillRect( fillrect, fill_brush );
	dc.SetTextAlign( TA_CENTER );
	dc.SelectObject(*TempFont);
	dc.SetBkColor( TColor::Black );
	dc.SetTextColor( TColor::LtRed );
	dc.TextOut( (int) (winrect.right*(465.0/winwidth)),
					(int) (winrect.bottom*(310.0/winheight)),
					temp );
}

void	TBusDialog::UpdateScreen( void )
{
	char temp[20];
	int i;

	if ( DoMatch )
		{
		ExpBus->DS1820[WhichROM]->ROMString( temp );
		SetDlgItemText( EB_ROMCODE, temp );
		SetDlgItemText( EB_ROMNAME, ExpBus->DS1820[WhichROM]->ROMName );

		for ( i = 0 ; i < 9 ; i++ )
			{
			if ( (i==2 || i==3) && (ExpBus->DS1820[WhichROM]->SPData[i]>=128) )
				sprintf( temp, "%d", ExpBus->DS1820[WhichROM]->SPData[i]-256 );
			else
				ConvertData( ExpBus->DS1820[WhichROM]->SPData[i], temp, 10 );
			SetDlgItemText( EB_READ_SPTLSB_D + i, temp );
			ConvertData( ExpBus->DS1820[WhichROM]->SPData[i], temp, 16 );
			SetDlgItemText( EB_READ_SPTLSB_H + i, temp );
			ConvertData( ExpBus->DS1820[WhichROM]->SPData[i], temp, 2 );
			SetDlgItemText( EB_READ_SPTLSB + i, temp );
			}
		temperature = ExpBus->DS1820[WhichROM]->ReadTemperature();
		}
	else
		{
		ExpBus->ROMString( temp );
		SetDlgItemText( EB_ROMCODE, temp );
		SetDlgItemText( EB_ROMNAME, "(No name, SKIP ROM selected)" );

		for ( i=0 ; i<9 ; i++ )
			{
			if ( (i==2 || i==3) && ( ExpBus->SPData[i]>=128 ) )
				sprintf( temp, "%d", ExpBus->SPData[i]-256 );
			else
				ConvertData( ExpBus->SPData[i], temp, 10 );
			SetDlgItemText( EB_READ_SPTLSB_D + i, temp );
			ConvertData( ExpBus->SPData[i], temp, 16 );
			SetDlgItemText( EB_READ_SPTLSB_H + i, temp );
			ConvertData( ExpBus->SPData[i], temp, 2 );
			SetDlgItemText( EB_READ_SPTLSB + i, temp );
			}
		temperature = ExpBus->ReadTemperature();
		}

	UpdateTemp();
}

void	TBusDialog::ConvertData( uchar val, char *code, int radix )
{
	int i;

	if ( radix==2 )
		{
		for ( i=0 ; i<8 ; i++ )
			{
			if ( val & (1<<(7-i)) )
				code[i] = '1';
			else
				code[i] = '0';
			}
		code[i] = '\0';
		}
	else
		if ( radix==16 )
			sprintf( code, "%02X", (int) val );
		else
			sprintf( code, "%d", (int) val );
}

void	TBusDialog::BadSetup( void )
{
	MessageBox( "Bad Setup File\nAborting" );
}

int	TBusDialog::ScanData( char* inBuffer )
{
	uchar tempROM[8];
	char hold;

	for ( int i=0 ; i<8 ; i++ )
		{
		hold = inBuffer[i*2];
		if ( hold >= '0' && hold <= '9' )
			tempROM[7-i] = (hold - '0')*16;
		else
			if ( hold >= 'A' && hold <= 'F' )
				tempROM[7-i] = (hold - 'A' + 0xA)*16;
			else
				return -1;

		hold = inBuffer[i*2+1];
		if ( hold >= '0' && hold <= '9' )
			tempROM[7-i] += hold - '0';
		else
			if ( hold >= 'A' && hold <= 'F' )
				tempROM[7-i] += hold - 'A' + 0xA;
			else
				return -1;
		}

	if ( strlen( inBuffer ) >= 17 )
		ExpBus->ROMNames->AddROMName( tempROM, inBuffer+17 );
	else
		return -1;

//	MessageBox( inBuffer+17, "ROM Name" );

	return 0;
}

void	TBusDialog::LoadSetup( void )
{
	char inBuffer[60];

	TOpenSaveDialog::TData SetupFileData(
		OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST,
		"Setup (*.set)|*.set|All Files (*.*)|*.*|" );
	if ( TFileOpenDialog( this, SetupFileData, 0, "Load Setup" )
		.DoExecute()==IDOK )
		{
		ifstream loadFile( SetupFileData.FileName );
		loadFile.getline( inBuffer, 60 );
		switch ( inBuffer[0] )
			{
			case '1':   SetPort1();
							break;
			case '2':   SetPort2();
							break;
			case '3':   SetPort3();
							break;
			case '4':   SetPort4();
							break;
			default:		BadSetup();
							return;
			}
		while ( loadFile.getline( inBuffer, 60 ) )
			if ( ScanData( inBuffer ) != 0 )
				{
				BadSetup();
				return;
				}

		CreateROMList();

		SendDlgItemMessage( EB_SEL_ROM_BOX, LB_SETCURSEL, WhichROM, 0 );

		char temp1[50];

		for ( int i=0 ; i<ExpBus->num_parts ; i++ )
			{
			ExpBus->ROMNames->FindROMName( ExpBus->DS1820[i]->ROMCode, temp1 );
			ExpBus->DS1820[i]->SetName( temp1 );
			}

		UpdateScreen();
		}
}

void	TBusDialog::SaveSetup( void )
{
	TOpenSaveDialog::TData SetupFileData(
		OFN_PATHMUSTEXIST,
		"Setup (*.set)|*.set|All Files (*.*)|*.*|" );
	if ( TFileSaveDialog( this, SetupFileData, 0, "Save Setup" )
		.DoExecute()==IDOK )
		{
		ofstream saveFile( SetupFileData.FileName );
		saveFile << ExpBus->numLPT << '\n';

		char str_temp[10];

		for ( int i=0 ; i<ExpBus->ROMNames->num_names ; i++ )
			{
			for ( int j=0 ; j<8 ; j++ )
				{
				sprintf( str_temp, "%02X",
					(int) ExpBus->ROMNames->ROMCodeList[i][7-j] );
				saveFile << str_temp;
				}
			saveFile << ' ' << ExpBus->ROMNames->ROMNameList[i] << '\n';
			}
		}
}

void	TBusDialog::SetPort1( void )
{
	if ( ExpBus->portLPT[0] != 0 )
		{
		CheckDlgButton( EB_LPT1 + ExpBus->numLPT - 1, 0 );
		ExpBus->SetPort( 1 );
		CheckDlgButton( EB_LPT1, 1 );
		}
	else
		CheckDlgButton( EB_LPT1, 2 );
}

void	TBusDialog::SetPort2( void )
{
	if ( ExpBus->portLPT[1] != 0 )
		{
		CheckDlgButton( EB_LPT1 + ExpBus->numLPT - 1, 0 );
		ExpBus->SetPort( 2 );
		CheckDlgButton( EB_LPT2, 1 );
		}
	else
		CheckDlgButton( EB_LPT2, 2 );
}

void	TBusDialog::SetPort3( void )
{
	if ( ExpBus->portLPT[2] != 0 )
		{
		CheckDlgButton( EB_LPT1 + ExpBus->numLPT - 1, 0 );
		ExpBus->SetPort( 3 );
		CheckDlgButton( EB_LPT3, 1 );
		}
	else
		CheckDlgButton( EB_LPT3, 2 );
}

void	TBusDialog::SetPort4( void )
{
	if ( ExpBus->portLPT[3] != 0 )
		{
		CheckDlgButton( EB_LPT1 + ExpBus->numLPT - 1, 0 );
		ExpBus->SetPort( 4 );
		CheckDlgButton( EB_LPT4, 1 );
		}
	else
		CheckDlgButton( EB_LPT4, 2 );
}

DEFINE_RESPONSE_TABLE1( TMainGadgetWindow, TToolBox )
	EV_COMMAND( BUS_EXP_BM, HandleBusButton ),
	EV_COMMAND( CS_SIM_BM, HandleControlButton ),
	EV_COMMAND( EXIT_BM, HandleExitButton ),
	EV_COMMAND( HELP_BM, HandleHelpButton ),
END_RESPONSE_TABLE;

TMainGadgetWindow::TMainGadgetWindow( void )
	: TToolBox( 0, 2, 2 )
{
	TMargins mg;
	TButtonGadget* bg;

	cs = (TSimWindow*) NULL;
	be = (TBusDialog*) NULL;
	hlp = (THelpDialog*) NULL;

	mg.Units = TMargins::LayoutUnits;
	mg.Left = 5;
	mg.Right = 5;
	mg.Top = 5;
	mg.Bottom = 5;

	cs_bg = new TButtonGadget( CS_SIM_BM, CS_SIM_BM );
	cs_bg->SetMargins( mg );
	Insert( *cs_bg );

	be_bg = new TButtonGadget( BUS_EXP_BM, BUS_EXP_BM );
	be_bg->SetMargins( mg );
	Insert( *be_bg );

	hlp_bg = new TButtonGadget( HELP_BM, HELP_BM );
	hlp_bg->SetMargins( mg );
	Insert( *hlp_bg );

	bg = new TButtonGadget( EXIT_BM, EXIT_BM );
	bg->SetMargins( mg );
	Insert( *bg );

	MainBus = new ThermBus( 0 );

	del_cs = TRUE;
	del_be = TRUE;
	del_hlp = TRUE;
}

TMainGadgetWindow::~TMainGadgetWindow( void )
{
}

BOOL	TMainGadgetWindow::CanClose( void )
{
	int saveNo = 0;

	while ( SetupDirty==1 && saveNo==0 )
		switch
			( MessageBox( "Setup data has changed.\nDo you want to save?",
				"Save Setup Data", MB_YESNOCANCEL | MB_ICONQUESTION ) )
			{
			case IDYES:		if ( SaveSetup() )
									SetupDirty = 0;
								break;
			case IDNO:		saveNo = 1;
								break;
			case IDCANCEL:	return FALSE;
			}

	return MessageBox( "Do you want to exit?",
		"DS1820 Demo Program", MB_YESNO | MB_ICONQUESTION )==IDYES;
}

BOOL	TMainGadgetWindow::SaveSetup( void )
{
	TOpenSaveDialog::TData SetupFileData(
		OFN_PATHMUSTEXIST,
		"Setup (*.set)|*.set|All Files (*.*)|*.*|" );
	if ( TFileSaveDialog( this, SetupFileData, 0, "Save Setup" )
		.DoExecute()==IDOK )
		{
		ofstream saveFile( SetupFileData.FileName );
		saveFile << MainBus->numLPT << '\n';

		char str_temp[10];

		for ( int i=0 ; i<MainBus->ROMNames->num_names ; i++ )
			{
			for ( int j=0 ; j<8 ; j++ )
				{
				sprintf( str_temp, "%02X",
					(int) MainBus->ROMNames->ROMCodeList[i][7-j] );
				saveFile << str_temp;
				}
			saveFile << ' ' << MainBus->ROMNames->ROMNameList[i] << '\n';
			}
		return TRUE;
		}
	else
		return FALSE;
}

void	TMainGadgetWindow::HandleControlButton( void )
{
	if ( del_cs && del_be )
		(cs = new TSimWindow( this, "Sim Window", MainBus,
			cs_bg, be_bg, &del_cs ))->Create();
}

void	TMainGadgetWindow::HandleBusButton( void )
{
	if ( del_cs && del_be )
		(be = new TBusDialog( this, EB_DLG, MainBus, cs_bg, be_bg, &del_be ))
			->Create();
}

void	TMainGadgetWindow::HandleHelpButton( void )
{
	if ( del_hlp )
		(hlp = new THelpDialog( this, HELP_DLG, hlp_bg, &del_hlp ))->Create();
}

TMainFrame::TMainFrame( const char* title )
	: TFrameWindow( 0, title, new TMainGadgetWindow(), TRUE )
{
	int xWindExt, yWindExt;

	xWindExt = GetSystemMetrics( SM_CXSCREEN );
	yWindExt = GetSystemMetrics( SM_CYSCREEN );

	Attr.X = xWindExt/2 - 110;
	Attr.Y = yWindExt/2 - 110;
	Attr.Style = WS_VISIBLE | WS_POPUPWINDOW | WS_CAPTION | WS_MINIMIZEBOX;
}

void TMainApp::InitMainWindow( void )
{
	EnableBWCC( TRUE );
	MainWindow = new TMainFrame( "DS1820 Demo" );
	MainWindow->SetIcon(this, ALARM_ICON);
}

int OwlMain( int, char** )
{
	SetupDirty = 0;
	return TMainApp().Run();
}
